!function($,D,window,document,undefined) {
  "use strict";

  D.behaviors.dismissibleMessageBar = {
    attach: function (context, settings) {

      /**
       * Retrieve DMB Notification Settings.
       *
       * @return {Array}
       *   Array of defined DMB Settings.
       */
      const getNotificationSettings = function () {
        if (typeof(settings.dmbNotifications) !== 'undefined' && settings.dmbNotifications) {
          return settings.dmbNotifications;
        }
        return null;
      };

      /**
       * Retrieve DMB Notification Entity data.
       *
       * @return {Array}
       *   Array of defined DMB Entities.
       */
      const getNotificationEntities = function () {
        if (typeof(settings.dmbNotificationEntities) !== 'undefined' && settings.dmbNotificationEntities) {
          return settings.dmbNotificationEntities;
        }
        return null;
      };

      /**
       * Close notification functionality
       * @param $notification
       *   Notification element to be closed.
       */
      const closeNotification = function ($notification) {
        let notificationId = $notification.attr('dmb-notification-id'),
          cookieExpiration = $notification.attr('dmb-cookie-expiration'),
          acceptedNotifications = $.cookie('dismissible_message_bar');

        if (!acceptedNotifications) {
          acceptedNotifications = notificationId;
        } else {
          acceptedNotifications = acceptedNotifications.split('.');
          if (!acceptedNotifications.includes(notificationId)) {
            acceptedNotifications.push(notificationId);
          }
          acceptedNotifications = acceptedNotifications.join('.');
        }

        // Set the notification cookie with updated values
        const setAcceptedCookie = function () {
          $.cookie('dismissible_message_bar', acceptedNotifications, {
            expires: Number(cookieExpiration),
            path: '/'
          });
        };

        // Only set cookie if cookie-off class doesn't exist.
        if(!$('.cookie-off').length) {
          setAcceptedCookie();
        }

        $notification.slideUp();
        return false;
      };

      /**
       * Inject Notifications into their respective wrappers.
       */
      const injectNotifications = function (notificationSettings, $notificationWrapper, requestData) {
        // Inject notifications.
        if (notificationSettings !== null && $notificationWrapper !== null) {

          let data = getNotificationEntities();
          let content = '';
          for (let key in data) {
            // Skip loop if the property is from prototype.
            if (!data.hasOwnProperty(key) || data[key].added === true) {
              continue;
            }

            if ($notificationWrapper.length > 0) {
              if (data[key].content !== null) {
                let showNotification = (data[key].sitewide === 1 || data[key].sitewide === "1");

                // Check content types.
                if (data[key].contentTypes !== null && data[key].contentTypes.length !== 0 && requestData.contentType !== '') {
                  if (data[key].contentTypes.includes(requestData.contentType)) {
                    showNotification = true;
                  }
                }

                // Check path limitations.
                if (data[key].sitewide !== null && (data[key].sitewide === 0 || data[key].sitewide === "0")) {
                  if (data[key].pathLimit !== null && window.location.pathname !== '') {
                    for (let paths in data[key].pathLimit) {
                      if (!data[key].pathLimit.hasOwnProperty(paths)) {
                        continue;
                      }

                      let searchPath = data[key].pathLimit[paths];
                      let isNot = false;
                      if (searchPath.indexOf("!") === 0) {
                        // This is a NOT path.
                        isNot = true;
                        searchPath = searchPath.slice(1);
                      }

                      if (searchPath === window.location.pathname) {
                        showNotification = !isNot;
                        break;
                      }
                      else {
                        if (searchPath.indexOf("*") !== -1) {
                          // This is a wildcard search. Replace the * with a .*
                          searchPath.replace(/\*/g, ".*");
                          if (window.location.pathname.search(searchPath) !== -1) {
                            showNotification = true;
                            break;
                          }
                        }
                      }
                    }
                  }
                }

                // Check date limitations
                if (showNotification) {
                  let nowDate = new Date();
                  if (data[key].startTime !== null && data[key].startTime !== "") {
                    let startDate = new Date(data[key].startTime);
                    if (startDate instanceof Date && !isNaN(startDate)) {
                      if (startDate.valueOf() > nowDate.valueOf()) {
                        // This notification should not appear yet.
                        showNotification = false;
                      }
                    }
                  }
                  if (showNotification && data[key].endTime !== null && data[key].endTime !== "") {
                    let endDate = new Date(data[key].endTime);
                    if (endDate instanceof Date && !isNaN(endDate)) {
                      if (nowDate.valueOf() > endDate.valueOf()) {
                        // This notification has passed.
                        showNotification = false;
                      }
                    }
                  }
                }

                if (showNotification) {
                  content += data[key].content;
                  data[key].added = true;
                }
              }
            }
          }

          if (content !== '') {
            $notificationWrapper.once('inject-dmb-notifications').append(content);
            $notificationWrapper.once('inject-dmb-notifications-trigger').trigger('injectDMBNotifications');
          }

        }

      };

      // Initialize notifications.
      let settingsObject = getNotificationSettings();
      let notificationSettingsArray = Object.values(settingsObject);
      for (let i in notificationSettingsArray) {
        let notificationSettings = notificationSettingsArray[i];
        let $notificationWrapper = null,
          $overlayWrapper = null,
          requestData = {
            contentType: '',
            contentPath: '',
            notificationType: '',
          };

        if (typeof(notificationSettings.notificationWrapper) !== 'undefined' && notificationSettings.notificationWrapper) {
          $notificationWrapper = $(notificationSettings.notificationWrapper);
        }

        if (typeof(notificationSettings.overlayWrapper) !== 'undefined' && notificationSettings.overlayWrapper) {
          $overlayWrapper = $(notificationSettings.overlayWrapper);
        }

        // These parameters have to be saved and passed into the handler.
        if (typeof(notificationSettings.contentType) !== 'undefined' && notificationSettings.contentType) {
          requestData.contentType = notificationSettings.contentType;
        }

        if (typeof(notificationSettings.contentPath) !== 'undefined' && notificationSettings.contentPath) {
          requestData.contentPath = notificationSettings.contentPath;
        }

        if (typeof(notificationSettings.notificationType) !== 'undefined' && notificationSettings.notificationType) {
          requestData.notificationType = notificationSettings.notificationType;
        }

        injectNotifications(notificationSettings, $notificationWrapper, requestData);

        if ($notificationWrapper !== null) {

          $notificationWrapper.on('injectDMBNotifications', function () {
            let $autoDismiss = $notificationWrapper.find('[dmb-auto-dismiss="1"]');
            $autoDismiss.each(function () {
              let time = $(this).attr('dmb-dismiss-time') * 1000;
              let $notification = $(this);
              setTimeout(function () {
                closeNotification($notification);
              }, time);
            });

            $('.dmb-notification .close-dmb-notification').once().on('click touchstart', function (e) {
              e.preventDefault();
              closeNotification($(this).parent());
            });
          });
        }

      }

    }
  };

}(jQuery,Drupal,window,document,'');
